/*  28 November 2007. UIAV Venice mobile services workshop. 
 By Nicholas Zambetti
 Example of Dynamic Live-Rendered Animation (both linear and eased)
 Now with easeIn to simulate bounce
 */

// declare the variables that describe a box
int boxX;
int boxY;
int boxWidth;
int boxHeight;

int BOUNCE_YMIN;
int BOUNCE_YMAX;

/////////////////////////////////////////////////////////////////////////////////////////////
// Setup, Executes Once When Started, Prepares Program to Run (Logic Initialization)
/////////////////////////////////////////////////////////////////////////////////////////////

void setup()
{
  framerate(10);
  rectMode(CENTER);
  
  BOUNCE_YMIN = height - 10;
  BOUNCE_YMAX = height / 2;

  boxX = width / 2;
  boxY = BOUNCE_YMIN;
  boxWidth = 10;
  boxHeight = 10;
}

/////////////////////////////////////////////////////////////////////////////////////////////
// Draw, Executes Forever, Provides User Feedback (Logic Repetition)
/////////////////////////////////////////////////////////////////////////////////////////////

boolean goingUp = true;

void draw()
{
  // draw the box
  background(100);                        // draw the background (clears the screen)
  rect(boxX, boxY, boxWidth, boxHeight);  // draw the box using variables

  // ease the bounce
  if(goingUp){
    boxY = easeOut(boxY, BOUNCE_YMAX, 2);
    if(boxY == BOUNCE_YMAX){
      goingUp = false;
    }
  }
  else{
    boxY = easeIn(boxY, BOUNCE_YMIN, 600);
    if(boxY == BOUNCE_YMIN){
      goingUp = true;
    }
  }
}

/////////////////////////////////////////////////////////////////////////////////////////////
// Animation Functions, Provide next value given the current value, target value, and speed
/////////////////////////////////////////////////////////////////////////////////////////////

// linear animation function (relative)
// e.g. easeIn(x, 100, 2);
int animateTo(int current, int target, int speed)
{
  int next = current;          // initially our next step is where we currently are
  if(target < next){           // if we need to make our value smaller...
    next = next - speed;       // make it smaller
    next = max(next, target);  // ensure that we didn't make it smaller than our target value
  } 
  else if(next < target){      // if we need to make our value bigger...
    next = next + speed;       // make it bigger
    next = min(next, target);  // ensure that we didn't make it bigger than our target value
  }
  return next;                 // return our next value
}

// ease out animation function (relative)
// e.g. easeIn(x, 100, 10);
int easeOut(int currentOriginal, int targetOriginal, int speed)
{
  // make values large for math
  int current = currentOriginal * 1000; // make current value large for math (e.g. 2 becomes 2000)
  int target = targetOriginal * 1000;   // make target value large for math  (e.g. 33 becomes 33000)

  // do math to calculate our next value
  int change = target - current;     // find out how much change there is (e.g. 33000 - 2000 = 31000)
  int changeLittle = change / speed; // make the change a little change   (e.g. 31000 / 4 = 7750)
  int next = current + changeLittle; // change the current value a little (e.g. 2000 + 7750 = 9750)

  // make next value small for screen
  next = next / 1000;

  // if our little change was so little that we didn't move...
  if(next == currentOriginal){
    next = targetOriginal; // our next step is our target
  }

  return next; // return our next value (e.g. 9750 / 1000 = 9, remember that we started with 2)
}

// eased in animation function (relative), somewhat hack but ok for simple animation
// e.g. easeIn(x, 100, 900);
int easeIn(int current, int target, int speed)
{
  int change = target - current; // record how far we are away
  int next = current;            // initially our next step is where we currently are
  if(target < next){             // if we need to make our value smaller...
    change = target - change;    // flip how far we are away
    speed += change;             // divide by zeroes are bad
    speed /= change;             // divide
    next = next - speed;         // make it smaller
    next = max(next, target);    // ensure that we didn't make it smaller than our target value
  } 
  else if(next < target){      // if we need to make our value bigger...
    speed += change;           // divide by zeroes are bad
    speed /= change;           // divide
    next = next + speed;       // make it bigger
    next = min(next, target);  // ensure that we didn't make it bigger than our target value
  }
  return next;                 // return our next value
}
